//*****************************************************************************
// File Name            : iobus2wb.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze_MAC Iobus to Wishbone BUS 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2011/12/17
//*****************************************************************************
module iobus2wb (
                 
Clk, Reset, IO_Ready, IO_Addr_Strobe, IO_Read_Strobe, IO_Write_Strobe,  IO_Read_Data, IO_Address, IO_Byte_Enable, IO_Write_Data,
clk_o, rst_o, cyc_o, stb_o, adr_o, sel_o, we_o, dat_o, dat_i, ack_i //wb 
);
input Clk;
input Reset;
input IO_Addr_Strobe;
input IO_Read_Strobe;
input IO_Write_Strobe;
input [31 : 0] IO_Address;
input [3 : 0] IO_Byte_Enable;
input [31 : 0] IO_Write_Data;
output [31 : 0] IO_Read_Data;
output IO_Ready;
//
output         clk_o;         // clock
output         rst_o;         // reset (asynchronous active low)
output         cyc_o;         // cycle
output         stb_o;         // strobe
output  [31:0]  adr_o;         // address adr_o[1]
output  [3:0]  sel_o;         // byte_sel
output         we_o;          // write enable
output  [31:0] dat_o;         // data output
input   [31:0] dat_i;         // data input
input          ack_i;         // normal bus termination
//

reg [31:0] IO_Read_Data;
reg IO_Ready;
   
reg         cyc_o;         // cycle
reg         stb_o;         // strobe
reg  [31:0]  adr_o;         // address adr_o[1]
reg  [3:0]  sel_o;         // byte_sel
reg         we_o;          // write enable
reg  [31:0] dat_o;         // data output
//
assign    clk_o = Clk; 
assign    rst_o = Reset;

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    cyc_o <= 1'b0;
  else 
    if ((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1))
       cyc_o <=  1'b1 ;
    else if (ack_i==1'b1)
       cyc_o <=  1'b0 ;
    else
       cyc_o <=  cyc_o ;
    
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    stb_o <= 1'b0;
  else 
    if  ((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1))
       stb_o <=  1'b1 ;
    else if (ack_i==1'b1)
       stb_o <=  1'b0 ;
    else
       stb_o <=  stb_o ;
   
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    we_o <= 1'b0;
  else 
    if (IO_Write_Strobe==1'b1)
       we_o <=  1'b1 ;
    else if (ack_i==1'b1)
       we_o <=  1'b0 ;
    else
       we_o <=  we_o ;


always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    adr_o <= 32'h00000000;
  else 
    if ((IO_Address[31:29]==3'b110)&&(IO_Addr_Strobe==1'b1))
       adr_o <= IO_Address[31:0]  ;
    else
       adr_o <=  adr_o ;
   
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    sel_o <= 4'b0000;
  else 
    if ((IO_Address[31:29]==3'b110)&&(IO_Addr_Strobe==1'b1))
       sel_o <=  IO_Byte_Enable  ;
    else
       sel_o <= sel_o ;

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Ready <= 1'b1;
  else
    if ((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1))
      IO_Ready <= 1'b0;
    else if (ack_i==1'b1)      
      IO_Ready <=  1'b1 ;
    else
      IO_Ready <= IO_Ready ;

       

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    dat_o <= 32'h00000000;
  else 
    if ((IO_Address[31:29]==3'b110)&&(IO_Write_Strobe==1'b1))
       dat_o <= IO_Write_Data;  
    else
       dat_o <= dat_o ;
        
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Read_Data <= 32'h00000000;
  else 
    if ((IO_Address[31:29]==3'b110)&&(ack_i==1'b1))
      IO_Read_Data <= dat_i;
    else
      IO_Read_Data <= IO_Read_Data ;
        
endmodule