module wb_vram(
  clk_i, rst_i, w_cyc_i, w_stb_i, w_adr_i, w_we_i, w_dat_i, w_ack_o,
  r_cyc_i, r_stb_i, r_adr_i, r_dat_o, r_ack_o
               
);

  //
  // Inputs & outputs
  //
  input         clk_i;         // clock
  input         rst_i;         // reset (asynchronous active low)
  //write
  input         w_cyc_i;         // cycle
  input         w_stb_i;         // strobe
  input  [15:0] w_adr_i;         // address adr_i[1]
  input         w_we_i;          // write enable
  input  [ 31:0] w_dat_i;         // data output
  output        w_ack_o;         // normal bus termination

  //read
  input         r_cyc_i;         // cycle
  input         r_stb_i;         // strobe
  input  [15:0] r_adr_i;         // address adr_i[1]
  output [ 31:0] r_dat_o;         // data input
  output        r_ack_o;         // normal bus termination

  //

  reg           w_ack_o;   
  reg           r_ack_o;         
  //
  // perform parameter check
  //
   vram_blk vram (
  .clka(clk_i), // input clka
  .wea(w_we_i), // input [0 : 0] wea
  .addra(w_adr_i[14:2]), // input [13 : 0] addra
  .dina( w_dat_i), // input [31 : 0] dina
  .clkb(clk_i), // input clkb
  .addrb(r_adr_i[14:2]), // input [13 : 0] addrb
  .doutb(r_dat_o) // output [31 : 0] doutb
);


always @ (posedge clk_i or negedge rst_i )
  if (rst_i==1'b0)
    w_ack_o <= 1'b0;
  else
    w_ack_o <= w_stb_i;
   
always @ (posedge clk_i or negedge rst_i )
  if (rst_i==1'b0)
    r_ack_o <= 1'b0;
  else
    r_ack_o <= r_stb_i;

 
endmodule

